/*
 *	File:	lcd_2x16.c
 *	Date:	08.01.2011
 */


#include "main.h"
#include "mcu_gpio.h"

const uint8_t lcd_2x16_decode[] =
{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E',
		'F' };

enum
{
	LCD_2X16_RESET = 0x30, // Команда сброса ЖКД
	LCD_2X16_4_BIT_BUS = 0x20, // 4-х разрядная шина данных
	LCD_2X16_8_BIT_BUS = 0x30, // 8-и разрядная шина данных
	LCD_2X16_LINE_ONE = 0x20, // Одна строка
	LCD_2X16_LINES_TWO = 0x28, // Две строки
	LCD_2X16_FONT_5X8 = 0x20, // Шрифт 5x8 точек
	LCD_2X16_FONT_5X10 = 0x24, // Шрифт 5x10 точек
	LCD_2X16_DISPLAY_CLEAR = 0x01, // Очистка дисплея
	LCD_2X16_DISPLAY_HOME = 0x02, // Установка нулевой позиции курсора и дисплея
	LCD_2X16_DISPLAY_ON = 0x0C, // Включить дисплей. Комб. с CURSOR_ON и CURSOR_BLINK
	LCD_2X16_DISPLAY_RIGHT = 0x1C, // Сдвинуть дисплей вправо
	LCD_2X16_DISPLAY_LEFT = 0x18, // Сдвинуть дисплей влево
	LCD_2X16_DISPLAY_SHIFT = 0x05, // Комб. с SHIFTDIR_RIGHT и SHIFTDIR_LEFT
	LCD_2X16_CURSOR_ON = 0x0A, // Включить курсор. Комб. с DISPLAY_ON и CURSOR_BLINK
	LCD_2X16_CURSOR_BLINK = 0x09, // Комб. с DISPLAY_ON и CURSOR_ON
	LCD_2X16_CURSOR_RIGHT = 0x14, // Сдвинуть курсор вправо
	LCD_2X16_CURSOR_LEFT = 0x10, // Сдвинуть курсор влево
	LCD_2X16_SHIFT_RIGHT = 0x06, // Комб. с DISPLAY_SHIFT
	LCD_2X16_SHIFT_LEFT = 0x04, // Комб. с DISPLAY_SHIFT
	LCD_2X16_CGRAM_ADDR = 0x40, // Установка адреса CGRAM
	LCD_2X16_DDRAM_ADDR = 0x80, // Установка адреса DDRAM
	LCD_2X16_BUSY_FLAG = 0x80, // Флаг занятости
	LCD_2X16_NEXT_LINE = 0x40, // Адрес новой строки
	LCD_2X16_COMMAND = 0x01,
	LCD_2X16_DATA = 0x00
} lcd_2x16_command;

//-----------------------------------------------------------------------------
void lcd_2x16_write(uint8_t data)
{
	if (data & BIT(7))
		PIN_ON(LCD_2X16_DB7);

	if (data & BIT(6))
		PIN_ON(LCD_2X16_DB6);

	if (data & BIT(5))
		PIN_ON(LCD_2X16_DB5);

	if (data & BIT(4))
		PIN_ON(LCD_2X16_DB4);

	PIN_ON(LCD_2X16_E);
	delay_ms(10);
	PIN_OFF(LCD_2X16_E);

	PIN_OFF(LCD_2X16_DB7);
	PIN_OFF(LCD_2X16_DB6);
	PIN_OFF(LCD_2X16_DB5);
	PIN_OFF(LCD_2X16_DB4);
}

//-----------------------------------------------------------------------------
void lcd_2x16_write_command(uint8_t data)
{
	PIN_OFF(LCD_2X16_RS);

	lcd_2x16_write(data);
	lcd_2x16_write(data << 4);
}

//-----------------------------------------------------------------------------
void lcd_2x16_write_data(uint8_t data)
{
	PIN_ON(LCD_2X16_RS);

	lcd_2x16_write(data);
	lcd_2x16_write(data << 4);
}

//-----------------------------------------------------------------------------
void lcd_2x16_set_position(uint8_t row, uint8_t colum)
{
	uint8_t position = LCD_2X16_DDRAM_ADDR;

	if (row > 0)
	{
		position |= LCD_2X16_NEXT_LINE;
	}

	lcd_2x16_write_command(position | colum);
}

//-----------------------------------------------------------------------------
void lcd_2x16_print_char(uint8_t data)
{
	if (data == '\n')
	{
		lcd_2x16_write_command(LCD_2X16_DDRAM_ADDR | LCD_2X16_NEXT_LINE);
	}
	else
	{
		lcd_2x16_write_data(data);
	}
}

//-----------------------------------------------------------------------------
void lcd_2x16_print_string(uint8_t * string)
{
	while (*string != 0x00)
	{
		lcd_2x16_print_char(*string++);
	}
}

//-----------------------------------------------------------------------------
void lcd_2x16_print_hex_xx(uint8_t data)
{
	lcd_2x16_print_char(lcd_2x16_decode[(data >> 4) & 0x0F]);
	lcd_2x16_print_char(lcd_2x16_decode[data & 0x0F]);
}

//-----------------------------------------------------------------------------
void lcd_2x16_print_dec_xxx(uint8_t data)
{
	lcd_2x16_print_char(lcd_2x16_decode[(data / 100) & 0x0F]);
	lcd_2x16_print_char(lcd_2x16_decode[((data % 100) / 10) & 0x0F]);
	lcd_2x16_print_char(lcd_2x16_decode[((data % 100) % 10) & 0x0F]);
}

//-----------------------------------------------------------------------------
void lcd_2x16_print_dec_xx(uint8_t data)
{
	lcd_2x16_print_char(lcd_2x16_decode[((data % 100) / 10) & 0x0F]);
	lcd_2x16_print_char(lcd_2x16_decode[((data % 100) % 10) & 0x0F]);
}

//-----------------------------------------------------------------------------
void lcd_2x16_init(void)
{       
	PIN_CONFIGURATION(LCD_2X16_E);
	PIN_CONFIGURATION(LCD_2X16_RS);
        PIN_CONFIGURATION(LCD_2X16_RW);
	PIN_CONFIGURATION(LCD_2X16_DB7);
	PIN_CONFIGURATION(LCD_2X16_DB6);
	PIN_CONFIGURATION(LCD_2X16_DB5);
	PIN_CONFIGURATION(LCD_2X16_DB4);
        
 
	PIN_OFF(LCD_2X16_E);
	PIN_OFF(LCD_2X16_RS);
        PIN_ON(LCD_2X16_RW);
	PIN_OFF(LCD_2X16_DB7);
	PIN_OFF(LCD_2X16_DB6);
	PIN_OFF(LCD_2X16_DB5);
	PIN_OFF(LCD_2X16_DB4);

	delay_ms(15);

	lcd_2x16_write(LCD_2X16_RESET);
	delay_ms(5);

	lcd_2x16_write(LCD_2X16_RESET);
	delay_ms(1);

	lcd_2x16_write(LCD_2X16_RESET);

	lcd_2x16_write(LCD_2X16_4_BIT_BUS);
	lcd_2x16_write(LCD_2X16_4_BIT_BUS);
	lcd_2x16_write(0x80);

	lcd_2x16_write(0x00);
	lcd_2x16_write(0xC0);
	delay_ms(1);

	lcd_2x16_write(0x00);
	lcd_2x16_write(0x10);
	delay_ms(1);

	lcd_2x16_write(0x00);
	lcd_2x16_write(0x60);
	delay_ms(1);
}
